@@ -151,6 +151,9 @@ MainWindow::loginCompleted (AsyncTaskToken *token)
         this->showMsgBox ("Severe internal error");
 
         dbMain.clearUserPass ();
+        dbMain.clearContactsPass ();
+        dbMain.setTFAFlag (false);
+
         QTimer::singleShot (500, this, SLOT(onRecreateCookieJar()));
         return;
     }
@@ -175,6 +178,9 @@ MainWindow::loginCompleted (AsyncTaskToken *token)
         Q_WARN(QString("User login failed. Error string: %1").arg(strErr));
 
         dbMain.clearUserPass ();
+        dbMain.clearContactsPass ();
+        dbMain.setTFAFlag (false);
+
         QTimer::singleShot (500, this, SLOT(onRecreateCookieJar()));
 
         if (token->status == ATTS_LOGIN_FAIL_SHOWURL) {
@@ -193,8 +199,29 @@ MainWindow::loginCompleted (AsyncTaskToken *token)
             dbMain.ensureCache ();
         }
 
-        // Prepare then contacts
-        initContacts ();
+        bool bInitContacts = true;
+        QString contactsPass = strPass;
+        if (dbMain.getTFAFlag ()) {
+            if (!dbMain.getContactsPass (contactsPass)) {
+                connect(this, SIGNAL(sigMessageBoxDone(bool)),
+                        this, SLOT(onAppPassMsgBoxDone(bool)));
+
+                QObject *obj = getQMLObject ("MsgBox");
+                if (NULL != obj) {
+                    obj->setProperty ("inputText", "");
+                }
+                Q_ASSERT(obj);
+
+                showInputBox ("Enter app specific password for Google contacts");
+                bInitContacts = false;
+            }
+        }
+
+        if (bInitContacts) {
+            // Prepare the contacts
+            initContacts (contactsPass);
+        }
+
         // Prepare the inbox widget for usage
         initInbox ();
 
@@ -308,3 +335,86 @@ MainWindow::setPassword(const QString &strP)
                                    Q_ARG (QVariant, QVariant(strP)));
     } while (0); // End cleanup block (not a loop)
 }//MainWindow::setPassword
+
+void
+MainWindow::onTwoStepAuthentication(AsyncTaskToken *token)
+{
+    CacheDatabase &dbMain = Singletons::getRef().getDBMain ();
+    dbMain.setTFAFlag (true);
+
+    QObject *obj = getQMLObject ("MsgBox");
+    if (NULL == obj) {
+        gvApi.cancel (token);
+        Q_ASSERT(obj);
+        return;
+    }
+
+    inputBoxCtx = token;
+    connect(this, SIGNAL(sigMessageBoxDone(bool)),
+            this, SLOT(onPinMsgBoxDone(bool)));
+
+    obj->setProperty ("inputText", "");
+    showInputBox ("Enter security token");
+}//MainWindow::onTwoStepAuthentication
+
+void
+MainWindow::onPinMsgBoxDone(bool ok)
+{
+    disconnect(this, SIGNAL(sigMessageBoxDone(bool)),
+               this, SLOT(onPinMsgBoxDone(bool)));
+
+    AsyncTaskToken *token = (AsyncTaskToken *)inputBoxCtx;
+    inputBoxCtx = NULL;
+
+    QObject *obj = getQMLObject ("MsgBox");
+
+    do {// Begin cleanup block (not a loop)
+
+        if (NULL == obj) {
+            gvApi.cancel (token);
+            Q_ASSERT(obj);
+            break;
+        }
+
+        if (!ok) {
+            gvApi.cancel (token);
+            break;
+        }
+
+        token->inParams["user_pin"] = obj->property("inputText").toString();
+        gvApi.resumeTFALogin (token);
+        token = NULL;
+    } while (0); // End cleanup block (not a loop)
+
+    if (NULL != token) {
+        token->deleteLater ();
+    }
+}//MainWindow::onPinMsgBoxDone
+
+void
+MainWindow::onAppPassMsgBoxDone(bool ok)
+{
+    disconnect(this, SIGNAL(sigMessageBoxDone(bool)),
+               this, SLOT(onAppPassMsgBoxDone(bool)));
+
+    do {// Begin cleanup block (not a loop)
+        QObject *obj = getQMLObject ("MsgBox");
+        if (NULL == obj) {
+            Q_ASSERT(obj);
+            break;
+        }
+
+        if (!ok) {
+            Q_WARN("User refused to enter app specific pass for contacts");
+            break;
+        }
+
+        QString contactsPass = obj->property("inputText").toString();
+
+        CacheDatabase &dbMain = Singletons::getRef().getDBMain ();
+        dbMain.setContactsPass (contactsPass);
+
+        // Prepare the contacts
+        initContacts (contactsPass);
+    } while (0); // End cleanup block (not a loop)
+}//MainWindow::onAppPassMsgBoxDone
