@@ -479,30 +479,10 @@ MainWindow::initQML ()
         this, SLOT   (onSigText(const QString&,const QString&)));
     Q_ASSERT(rv);
     if (!rv) { exit(1); }
-    rv = connect (gObj, SIGNAL(sigVoicemail(QString)),
-                  this, SLOT(retrieveVoicemail(const QString &)));
-    Q_ASSERT(rv);
-    if (!rv) { exit(1); }
-    rv = connect (gObj, SIGNAL(sigVmailPlayback (int)),
-                  this, SLOT(onSigVmailPlayback (int)));
-    Q_ASSERT(rv);
-    if (!rv) { exit(1); }
     rv = connect (gObj, SIGNAL(sigSelChanged(int)),
                   this, SLOT(onRegPhoneSelectionChange(int)));
     Q_ASSERT(rv);
     if (!rv) { exit(1); }
-    rv = connect (gObj  , SIGNAL(sigInboxSelect(QString)),
-                  oInbox, SLOT(onInboxSelected(const QString &)));
-    Q_ASSERT(rv);
-    if (!rv) { exit(1); }
-    rv = connect (gObj  , SIGNAL(sigMarkAsRead(QString)),
-                  oInbox, SLOT(onSigMarkAsRead(const QString &)));
-    Q_ASSERT(rv);
-    if (!rv) { exit(1); }
-    rv = connect (gObj, SIGNAL(sigCloseVmail ()),
-                  this, SLOT(onSigCloseVmail ()));
-    Q_ASSERT(rv);
-    if (!rv) { exit(1); }
     rv = connect (gObj, SIGNAL(sigHide ()),
                   this, SLOT(onSigHide ()));
     Q_ASSERT(rv);
@@ -526,15 +506,6 @@ MainWindow::initQML ()
     Q_ASSERT(rv);
     if (!rv) { exit(1); }
 
-    rv = connect (gObj, SIGNAL(sigRefreshInbox()),
-                  oInbox, SLOT(refresh()));
-    Q_ASSERT(rv);
-    if (!rv) { exit(1); }
-    rv = connect (gObj, SIGNAL(sigRefreshAllInbox()),
-                  oInbox, SLOT(refreshFullInbox()));
-    Q_ASSERT(rv);
-    if (!rv) { exit(1); }
-
     rv = connect (gObj, SIGNAL(sigHaptic()),
                   this, SLOT(onBtnClickFroHapticFeedback()));
     Q_ASSERT(rv);
@@ -552,6 +523,11 @@ MainWindow::initQML ()
     if (!rv) { exit(1); }
 
     gObj = getQMLObject ("MsgBox");
+    if (NULL == gObj) {
+        Q_WARN("Could not get to MsgBox");
+        requestQuit ();
+        return;
+    }
     rv = connect (gObj, SIGNAL(sigOk()), this, SLOT(onSigMsgBoxOk()));
     Q_ASSERT(rv);
     if (!rv) { exit(1); }
@@ -559,6 +535,44 @@ MainWindow::initQML ()
     Q_ASSERT(rv);
     if (!rv) { exit(1); }
 
+    gObj = getQMLObject ("InboxPage");
+    if (NULL == gObj) {
+        Q_WARN("Could not get to InboxPage");
+        requestQuit ();
+        return;
+    }
+    rv = connect (gObj  , SIGNAL(sigInboxSelect(QString)),
+                  oInbox, SLOT(onInboxSelected(const QString&)));
+    Q_ASSERT(rv);
+    if (!rv) { exit(1); }
+    rv = connect (gObj, SIGNAL(sigVoicemail(QString)),
+                  this, SLOT(retrieveVoicemail(const QString&)));
+    Q_ASSERT(rv);
+    if (!rv) { exit(1); }
+    rv = connect (gObj, SIGNAL(sigVmailPlayback(int)),
+                  this, SLOT(onSigVmailPlayback(int)));
+    Q_ASSERT(rv);
+    if (!rv) { exit(1); }
+    rv = connect (gObj  , SIGNAL(sigMarkAsRead(QString)),
+                  oInbox, SLOT(onSigMarkAsRead(const QString &)));
+    Q_ASSERT(rv);
+    if (!rv) { exit(1); }
+    rv = connect (gObj, SIGNAL(sigRefreshInbox()),
+                  oInbox, SLOT(refresh()));
+    Q_ASSERT(rv);
+    if (!rv) { exit(1); }
+    rv = connect (gObj, SIGNAL(sigRefreshAllInbox()),
+                  oInbox, SLOT(refreshFullInbox()));
+    Q_ASSERT(rv);
+    if (!rv) { exit(1); }
+    rv = connect(gObj, SIGNAL(sigCloseVmail()), this, SLOT(onSigCloseVmail()));
+    Q_ASSERT(rv);
+    if (!rv) { exit(1); }
+    rv = connect(gObj, SIGNAL(sigDeleteInboxEntry(const QString &)),
+                 this, SLOT(onSigDeleteInboxEntry(const QString &)));
+    Q_ASSERT(rv);
+    if (!rv) { exit(1); }
+
     // Now do the settings page
     if (!connectSettingsSignals ()) {
         exit (1);
@@ -1352,3 +1366,37 @@ MainWindow::onBtnClickFroHapticFeedback()
     OsDependent &osd = Singletons::getRef().getOSD ();
     osd.onBtnClickFroHapticFeedback ();
 }//MainWindow::onBtnClickFroHapticFeedback
+
+void
+MainWindow::onSigDeleteInboxEntry(const QString &id)
+{
+   connect(this, SIGNAL(sigMessageBoxDone(bool)),
+           this, SLOT(onUserAllowedDelete(bool)));
+   inputBoxCtx = new QString(id);   
+   showMsgBox("Are you sure you want to delete this entry?");
+}//MainWindow::onSigDeleteInboxEntry
+
+void
+MainWindow::onUserAllowedDelete(bool ok)
+{
+   disconnect(this, SIGNAL(sigMessageBoxDone(bool)),
+              this, SLOT(onUserAllowedDelete(bool)));
+
+   QString *id = (QString *)inputBoxCtx;
+   inputBoxCtx = NULL;
+
+   if (NULL == id) {
+      Q_CRIT("Context = NULL!!");
+      return;
+   }
+
+   if (ok) {
+      Q_DEBUG("User confirmed deletion");
+      oInbox->onSigDeleteInboxEntry(*id);
+   } else {
+      Q_DEBUG("User canceled deletion");
+   }
+
+   delete id;
+}//MainWindow::onUserAllowedDelete
+
